/* outboxstojobs.h

 ***************************************************************************
 *                                                                         *
 *   This library is free software; you can redistribute it and/or         *
 *   modify it under the terms of the GNU Lesser General Public            *
 *   License as published by the Free Software Foundation; either          *
 *   version 2.1 of the License, or (at your option) any later version.    *
 *                                                                         *
 *   This library is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU     *
 *   Lesser General Public License for more details.                       *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the Free Software   *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston,                 *
 *   MA  02111-1307  USA                                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef OUTBOXSTOJOBS_H
#define OUTBOXSTOJOBS_H

/** @file outboxstojobs.h
 *
 * @short Standing Order related Outboxjobs and their C wrappers. */

#ifdef __cplusplus
#include <string>
#include <list>
#endif /* __cplusplus */

#include <openhbci/dllimport.h>
#include <openhbci/pointer.h>
#include <openhbci/customer.h>
#include <openhbci/user.h>
#include <openhbci/bank.h>
#include <openhbci/account.h>
#include <openhbci/outboxaccjobs.h>


#ifdef __cplusplus

namespace HBCI {

/**
 * This job retrieves all standing orders for one account.
 * @short Job that retrieves all standing orders for one Account
 * @author Fabian Kaiser<openhbci@derzach.de>
 */
class DLLIMPORT OutboxJobGetStandingOrders: public OutboxAccountJob {
private:
    Pointer<Job> _job;
    list<Pointer<StandingOrder> > _orders;
public:
    OutboxJobGetStandingOrders(Pointer<Customer> c,
                               Pointer<Account> a);
    ~OutboxJobGetStandingOrders();

    /**
     * @short Returns the list of standing orders that were retrieved.
     */
    const list<Pointer<StandingOrder> > &getOrders() const;

    bool createHBCIJobs(Pointer<MessageQueue> mbox, int n=0);
    bool evaluate();
    bool stillMessagesToSend(int nextMsg) const;
    bool commit(int msgNumber=HBCI_COMMIT_WHOLE_JOB);
    string description() const;
    JobProgressType type() const { return JOB_GET_STO; };
    list<int> resultCodes() const;
};

} /* namespace HBCI */
typedef struct HBCI::OutboxJobGetStandingOrders HBCI_OutboxJobGetStOs;
#else /* __cplusplus */
typedef struct HBCI_OutboxJobGetStOs HBCI_OutboxJobGetStOs;
#endif /* __cplusplus */


#ifdef __cplusplus

namespace HBCI {

/**
 * This job creates a new standing order.
 * @short Job that creates a new standing order for a Account at the bank
 * @author Fabian Kaiser<openhbci@derzach.de>
 */
class DLLIMPORT OutboxJobNewStandingOrder: public OutboxAccountJob {
private:
    Pointer<Job> _job;
    StandingOrder _newOrder;
public:
    OutboxJobNewStandingOrder(Pointer<Customer> c,
                              Pointer<Account> a,
                              const StandingOrder &newOrder);
    ~OutboxJobNewStandingOrder();

    bool createHBCIJobs(Pointer<MessageQueue> mbox, int n=0);
    bool evaluate();
    bool stillMessagesToSend(int nextMsg) const;
    bool commit(int msgNumber=HBCI_COMMIT_WHOLE_JOB);
    string description() const;
    JobProgressType type() const { return JOB_NEW_STO; };
    list<int> resultCodes() const;
};

} /* namespace HBCI */
typedef struct HBCI::OutboxJobNewStandingOrder HBCI_OutboxJobNewStO;
#else /* __cplusplus */
typedef struct HBCI_OutboxJobNewStO HBCI_OutboxJobNewStO;
#endif /* __cplusplus */


#ifdef __cplusplus

namespace HBCI {

/**
 * This job deletes an existing standing order.
 * @short Job that deletes an existing standing order
 * @author Fabian Kaiser<openhbci@derzach.de>
 */
class DLLIMPORT OutboxJobDeleteStandingOrder: public OutboxAccountJob {
private:
    Pointer<Job> _job;
    StandingOrder _order2Delete;
public:
    OutboxJobDeleteStandingOrder(Pointer<Customer> c,
                              Pointer<Account> a,
                              const StandingOrder &order2Delete);
    ~OutboxJobDeleteStandingOrder();

    bool createHBCIJobs(Pointer<MessageQueue> mbox, int n=0);
    bool evaluate();
    bool stillMessagesToSend(int nextMsg) const;
    bool commit(int msgNumber=HBCI_COMMIT_WHOLE_JOB);
    string description() const;
    JobProgressType type() const { return JOB_DELETE_STO; };
    list<int> resultCodes() const;
};

} /* namespace HBCI */
typedef struct HBCI::OutboxJobDeleteStandingOrder HBCI_OutboxJobDeleteStO;

#else /* __cplusplus */
typedef struct HBCI_OutboxJobDeleteStO HBCI_OutboxJobDeleteStO;
#endif /* __cplusplus */



#endif



